/*
 * laza.util - miscellaneous utility functions and prototype extensions
 *
 */

// Ensuring no 'console is undefined' errors happen

window.console = window.console || (function(){
	return {
		log: function(msg) {}
	};
})();

/*
 *	New constants
 */

var UNDEF = 'undefined',
	NOLINK = 'javascript:void(0)',
	LOCAL = location.protocol.indexOf('file:') === 0;
	
var LOCALSTORAGE = (function(){
		if (typeof window.localStorage === UNDEF) {
			return false;
		}
		try {
			localStorage.setItem('_t', UNDEF);
            localStorage.removeItem('_t');
			return true;
		} catch(e) {
			return false;
		}
	}());
	
	
var VEND = (function(ua) {
/*
	PC:
		IE 8: "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)"
		IE 9: "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)"
		IE 10: "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C)"
		Opera 12: "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15"
		Firefox 21: "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/20100101 Firefox/21.0"
		Chrome 27: "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36"
	Mac:
		Chrome 27: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36"
		Firefox 21: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/20100101 Firefox/21.0"
		Safari 6: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.1 Safari/536.26.14"
 */
		if (ua.indexOf('Trident') > 0) {
			return 'ms';
		} else if (ua.indexOf('AppleWebKit') > 0) {
			return 'webkit';
		} else if (ua.indexOf('Gecko') > 0) {
			return 'moz';
		} else if (ua.indexOf('Presto') > 0) {
			return 'o';
		} else if (ua.indexOf('Blink') > 0) {
			return 'webkit';
		}
		return '';		
	}(navigator.userAgent));

/*
 *	Extending prototypes
 */
 
String.wsp = [];
String.wsp[0x0009] = true;
String.wsp[0x000a] = true;
String.wsp[0x000b] = true;
String.wsp[0x000c] = true;
String.wsp[0x000d] = true;
String.wsp[0x0020] = true;
String.wsp[0x0085] = true;
String.wsp[0x00a0] = true;
String.wsp[0x1680] = true;
String.wsp[0x180e] = true;
String.wsp[0x2000] = true;
String.wsp[0x2001] = true;
String.wsp[0x2002] = true;
String.wsp[0x2003] = true;
String.wsp[0x2004] = true;
String.wsp[0x2005] = true;
String.wsp[0x2006] = true;
String.wsp[0x2007] = true;
String.wsp[0x2008] = true;
String.wsp[0x2009] = true;
String.wsp[0x200a] = true;
String.wsp[0x200b] = true;
String.wsp[0x2028] = true;
String.wsp[0x2029] = true;
String.wsp[0x202f] = true;
String.wsp[0x205f] = true;
String.wsp[0x3000] = true;

String.prototype.trim = function() { 
	var str = this + '', 
		j = str.length;
	if (j) {
		var ws = String.wsp, 
			i = 0;
		--j;
		while (j >= 0 && ws[str.charCodeAt(j)]) {
			--j;
		}
		++j;
		while (i < j && ws[str.charCodeAt(i)]) { 
			++i; 
		}
		str = str.substring(i, j);
	}
	return str;
};

String.prototype.trunc = function( n ) {
	var t = this + '';
	if (t.length <= n) {
		return t.toString();
	}
	var s = t.substring(0, n - 1), i = s.lastIndexOf(' ');
	return ((i > 6 && (s.length - i) < 20)? s.substring(0, i) : s) + '...';
};

String.prototype.startsWith = function( s ) {
	return (this + '').indexOf( s ) === 0;
};

String.prototype.endsWith = function( s ) {
	return (this + '').substring(this.length - s.length) === s;
};

String.prototype.capitalize = function() {
	return this.charAt(0).toUpperCase() + this.slice(1);
};

String.prototype.unCamelCase = function() {
	return this.replace(/([a-z])([A-Z])/g, "$1-$2").toLowerCase();
}
String.prototype.getExt = function() {
	var t = this + '', 
		i = t.lastIndexOf('.');
	return (i <= 0 || i >= t.length - 1)? '' : t.substring(i + 1).toLowerCase();
};

String.prototype.hasExt = function(x) {
	var t = (this + ''), 
		i = t.lastIndexOf('.');
	if (i >= 0) {
		t = t.substring(i + 1).toLowerCase();
		return (x + ',').indexOf(t + ',') >= 0;
	}
	return !1;
};

String.prototype.replaceExt = function( s ) {
	var t = this + '', 
		i = t.lastIndexOf('.');
	return (i <= 0)? t : (t.substring(0, i + 1) + s);  
};

String.prototype.fixExtension = function() {
	return (this + '').replace(/.gif$/gi, '.png').replace(/.tif+$/gi, '.jpg');
};

String.prototype.getDir = function() {
	var u = (this + '').split('#')[0];
	return u.substring(0, u.lastIndexOf('/') + 1);
};

String.prototype.getFile = function() {
	var u = (this + '').split('#')[0];
	return u.substring(u.lastIndexOf('/') + 1);
};

String.prototype.getRelpath = function(level) {
	var t = (this + ''), 
		i = t.lastIndexOf('#');
	if (i === -1) {
		i = t.length - 1;
	} else {
		i--;
	}
	for (; i >= 0; i--) {
		if (t[i] === '/' && (level--) === 0)
			break;
	}
	return t.substring(i + 1);
};

String.prototype.fixUrl = function() {
	var i, 
		j, 
		t = this + '';
	while ((i = t.indexOf('../')) > 0) {
		if (i === 1 || (j = t.lastIndexOf('/', i - 2)) === -1) {
			return t.substring(i + 3);
		}
		t = t.substring(0, j) + t.substring(i + 2);
	}
	return t;
};

String.prototype.fullUrl = function() {
	var t = this + '';
	if (!t.match(/^(http|ftp|file)/)) {
		t = window.location.href.getDir() + t;
	}
	return t.fixUrl();
};

String.prototype.cleanupHTML = function() {
	var htmlregex = [
		[ /<br>/gi, '\n' ],
		[ /\&amp;/gi, '&' ],
		[ /\&lt;/gi, '<' ],
		[ /\&gt;/gi, '>' ],
		[ /\&(m|n)dash;/gi , '-' ],
		[ /\&apos;/gi, '\'' ],
		[ /\&quot;/gi, '"' ]
	];
	var t = this + '';
	for (var i = htmlregex.length - 1; i >= 0; i--) {
		t = t.replace( htmlregex[i][0], htmlregex[i][1] );
	}
	return t; 
};

String.prototype.stripHTML = function(format) { 
	var s = (this + '');
	
	if (format) {
		s = s.cleanupHTML();
	}
	
	return s.replace(/<\/?[^>]+>/gi, ''); 
};

String.prototype.stripQuote = function() {
	return (this + '').replace(/\"/gi, '&quot;');
};

String.prototype.appendSep = function(s, sep) {
	return (this.length? (this + (sep || ' &middot; ')) : '') + s; 
};

String.prototype.rgb2hex = function() {
	var t = this + '';
	if (t.charAt(0) === '#' || t === 'transparent') {
		return t;
	}
	var n, r = t.match(/\d+/g), h = '';
	if ( r ) {
		for (var i = 0; i < r.length && i < 3; ++i) {
			n = parseInt( r[i], 10 ).toString(16);
			h += ((n.length < 2)? '0' : '') + n;
		}
		return '#' + h;
	}
	return 'transparent';
};

String.prototype.template = function( s ) {
	if (typeof s === UNDEF || !this) {
		return this;
	}
	if (!isNaN(parseFloat(s)) && isFinite(s)) {
		s = s + '';
	}
	var t = this + '';
	if (s.constructor === Array) {
		for (var i = 0; i < s.length; ++i) {
			t = t.replace( new RegExp('\\{' + i + '\\}', 'gi'), s[i] );
		}
	} else {
		t = t.replace( /\{0\}/gi, s );
	}
	return t;
};

String.prototype.getSearchTerms = function() {
	var t = this + '';
	
	if (t.indexOf('"') === -1) {
		return t.split(' ');
	} else {
		var a = [],
			i;
	
		do {
			if ((i = t.indexOf('"')) > 0) {
				a.push.apply(a, t.substring(0, i).split(' '));
			}
			t = t.substring(i + 1);
			i = t.indexOf('"');
			if (i < 0) {
				a.push(t);
				break;
			}
			a.push(t.substring(0, i));
			t = t.substring(i + 1);
			
		} while (t.length);
	
		return a;
	}
};

String.prototype.objectify = function() {
	if (!this || !this.length) {
		return this;
	}
	var t = this + '';
	if (t.charAt(0) === '?' || t.charAt(0) === '#') {
		t = t.substring(1);
	}
	var r = {}, o, os = t.split('&');
	for (var i = 0, l = os.length; i < l; ++i) {
		o = os[i].split('=');
		if (o.length > 1) {
			r[o[0]] = decodeURIComponent(o[1]);
		}
	}
	return r;
};

// Test if 's' is in the string
String.prototype.testIn = function(s) {
	if (typeof s !== 'string') {
		s = s + '';
	}
	return (new RegExp(this, 'i')).test(s);
};

// Testing exact match
String.prototype.testExactMatch = function(s) {
	if (s.constructor !== Array) {
		return this == s + '';
	} else {
		for (var i = 0, l = s.length; i < l; ++i) {
			if (this == s[i]) {
				return true;
			}
		}
	}
	return false;
};

// Ignore case but exact match
String.prototype.testMatch = function(s) {
	var t = this.toLowerCase(); 
	if (s.constructor !== Array) {
		return t == (s + '').toLowerCase();
	} else {
		for (var i = 0, l = s.length; i < l; ++i) {
			if (t == s[i].toLowerCase()) {
				return true;
			}
		}
	}
	return false;
};

// Creating hash code
String.prototype.hashCode = function(){
	for (var h = 0, i = 0, l = this.length; i < l; ++i) {
		h = (h << 5) - h + this.charCodeAt(i);
		h &= h;
	}
	return h;
}


Math.minMax = function(a, b, c) {
	b = (isNaN(b))? parseFloat(b) : b;
	return  (b < a)? a : ((b > c)? c : b); 
};


/*
 *	Touch mode detection
 *	Chrome >= 23, MSIE >= 10 :: dynamic
 *	all other browsers pre-initialized 
*/

var	TOUCH_START = 'touchstart',
	TOUCH_MOVE = 'touchmove',
	TOUCH_END = 'touchend',
	TOUCH_CANCEL = 'touchcancel';


(function($, d) {

	var setMsEvents = function() {
			if (window.navigator.pointerEnabled) {
				TOUCH_START = 'pointerdown';
				TOUCH_MOVE = 'pointermove';
				TOUCH_END = 'pointerup';
				TOUCH_CANCEL = 'pointercancel';
			} else {
				TOUCH_START = 'MSPointerDown';
				TOUCH_MOVE = 'MSPointerMove';
				TOUCH_END = 'MSPointerUp';
				TOUCH_CANCEL = 'MSPointerCancel';
			}
		},
		scrollbarWidth = function() {
			var div = document.createElement("div");
			div.style.cssText = 'width:100px;height:100px;overflow:scroll !important;position:absolute;top:-9999px';
			document.body.appendChild(div);
			var result = div.offsetWidth - div.clientWidth;
			document.body.removeChild(div);
			return result;
		};
	
	if (/Trident/.test(navigator.userAgent)) {
		d.touchEnabled = (typeof navigator['maxTouchPoints'] !== 'undefined' && navigator.maxTouchPoints); // || /IEMobile/.test(navigator.userAgent);
		setMsEvents();
	} else if (/Edge/.test(navigator.userAgent)) {
		d.touchEnabled = (scrollbarWidth() == 0);
		setMsEvents();
	} else {
		if (/(Chrome|CriOS)/.test(navigator.userAgent)) {
			d.touchEnabled = /Mobile/.test(navigator.userAgent) || Object.prototype.hasOwnProperty.call(window, 'ontouchstart'); 
		} else {
			d.touchEnabled = 'ontouchstart' in window;
		}
	}
	
	var TRANSFORM = (VEND === 'moz')? 'transform' : ('-' + VEND + '-transform');	
	// (VEND === 'webkit')? 'transform' : VEND[0].toUpperCase() + VEND.substring(1)
	
	$.fn.translate = function(x, y, cls) {
		return this.each(function() {
			if (cls) {                                    
				$(this).addClass(cls).data('tr_cls', cls);
			}
			this.style[TRANSFORM] = 'translate(' + (x || 0) + 'px,' + (y || 0) + 'px)';
			$(this).data({
				tr_x: x,
				tr_y: y
			});
		});
	};
	
	$.fn.translateToPos = function() {
		return this.each(function() {
			var t = $(this),
				x = t.data('tr_x') || 0, 
				y = t.data('tr_y') || 0;
			if (x || y) {
				var pos = t.position(),
					cls = t.data('tr_cls');
				if (cls) {
					t.removeClass(cls);
				}
				this.style[TRANSFORM] = 'translate(0,0)';
				t.removeData('tr_x tr_y').css({
					left: pos.left,
					top: pos.top
				});
			}
		});
	};

	
})(jQuery, document);

/*! jQuery requestAnimationFrame - v0.1.2 - 2013-04-15
* https://github.com/gnarf37/jquery-requestAnimationFrame
* Copyright (c) 2013 Corey Frang; Licensed MIT */

(function($) {

	// requestAnimationFrame polyfill adapted from Erik Möller
	// fixes from Paul Irish and Tino Zijdel
	// http://paulirish.com/2011/requestanimationframe-for-smart-animating/
	// http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating
	
	
	var animating,
		lastTime = 0,
		vendors = ['webkit', 'moz'],
		requestAnimationFrame = window.requestAnimationFrame,
		cancelAnimationFrame = window.cancelAnimationFrame;
	
	for(; lastTime < vendors.length && !requestAnimationFrame; lastTime++) {
		requestAnimationFrame = window[ vendors[lastTime] + "RequestAnimationFrame" ];
		cancelAnimationFrame = cancelAnimationFrame ||
			window[ vendors[lastTime] + "CancelAnimationFrame" ] || 
			window[ vendors[lastTime] + "CancelRequestAnimationFrame" ];
	}
	
	function raf() {
		if (animating) {
			requestAnimationFrame( raf );
			jQuery.fx.tick();
		}
	}
	
	if (requestAnimationFrame) {
		// use rAF
		window.requestAnimationFrame = requestAnimationFrame;
		window.cancelAnimationFrame = cancelAnimationFrame;
		jQuery.fx.timer = function( timer ) {
			if ( timer() && jQuery.timers.push( timer ) && !animating ) {
				animating = true;
				raf();
			}
		};
	
		jQuery.fx.stop = function() {
			animating = false;
		};
	} else {
		// polyfill
		window.requestAnimationFrame = function( callback, element ) {
			var currTime = new Date().getTime(),
				timeToCall = Math.max( 0, 16 - ( currTime - lastTime ) ),
				id = window.setTimeout( function() {
					callback( currTime + timeToCall );
				}, timeToCall );
			lastTime = currTime + timeToCall;
			return id;
		};
	
		window.cancelAnimationFrame = function(id) {
			clearTimeout(id);
		};
		
	}

}( jQuery ));


/*
 *	Translate function
 *	returns a translated key, or the default value (if exists), or the key itself de-camelcased
*/

var translate = function(key, def) {
	
	key = key.trim();
	
	if (typeof Texts !== UNDEF) {
		if (Texts.hasOwnProperty(key)) {
			return Texts[key];
		}
	}
	
	if (typeof def !== UNDEF) {
		// Using the default
		if (DEBUG && console) {
			console.log('Using default translation: '+key+'='+def);
		}
		return def;
	}
	
	if (DEBUG && console) {
		console.log('Missing translation: '+key);
	}
	var s = key.replace(/([A-Z])/g, ' $1').toLowerCase();
	s[0] = s.charAt(0).toUpperCase();
	return s;
};

/* 
 *	Simpler method 
 *	text = getKeys('key1,key2,key3', [defaults])
*/

var getKeys = function(keys, def) {
	
	var t = {}, i, k = keys.split(','), l = k.length;
	
	for (i = 0; i < l; i++) {
		t[k[i]] = translate(k[i], def[k]);
	}
	
	return t;
};

/*
 *	Finds translation for each key in def Object
 *	def should contain only elements that need translation
*/

var getTranslations = function(def) {
	
	var t = {}, k;
	
	for (k in def) {
		if (typeof def[k] === 'object') {
			t[k] = getTranslations(def[k]);
		} else {
			t[k] = translate(k, def[k]);
		}
	}
	
	return t;
};

// Printing an image with caption

var printImage = function(src, title, caption) {
	
	if (!src) {
		return;
	}
	
	var pw = window.open('about:blank', 'print', 'location=no,status=no,titlebar=no');
	
	pw.document.open();
	pw.document.write('<!DOCTYPE html>\n<html>\n<head>\n<meta charset="utf-8">\n<title>' + (title || 'Print') + '</title>' +
		'\n<script>printOut=function(){window.print();window.close();}</scr' + 'ipt>' + 
		'\n<style>body{margin:0;padding:0;text-align:center;overflow:hidden;}\nimg{display:block;width:100%;height:auto;}</style>' +
		'\n</head>\n<body onLoad="setTimeout(printOut,100)">' +
		'<img src="' + src + '">' + (caption || '') + 
		'</body>\n</html>');
	pw.document.close();
};


/*
 * Simple decryption
 */

var xDecrypt = function(c) {
	
	if (!typeof this === 'String') {
		return '';
	}
	var xs = [0x93,0xA3,0x57,0xFE,0x99,0x04,0xC6,0x17];
	var cl = c.length,
		sl = Math.ceil(cl / 8) * 5;
	var src = new Array(sl), r = '', i, j = 0, k, v;
	for (i = 0; i < sl; i++) {
		src[i] = 0;
	}
	for (i = 0; i < cl; i++) {
		if ((v = c.charCodeAt(i) - 0x30) > 9) {
			v -= 7;
		}
		v <<= 11 - j % 8;
		k = Math.floor(j / 8);
		if (k < sl) {
			src[k] |= v >> 8;
			if (++k < sl)
				src[k] |= v & 0xff;
		}
		j += 5;
	}
	for (i = 0; i < sl; i++) {
		src[i] ^= xs[i % 8];
	}
	sl = src[0] | (src[1] << 8);
	for (v = 0, i = 4; i < sl; i++) {
		r += String.fromCharCode(src[i]);
		v += src[i];
	}
	if (v != ((src[2] & 0xff) | (src[3] << 8)))
		r = '';
	return r;
};


/*	
 *	Debugging functions
 */

var log = function() {};
 
(function($) {
		
	// log: logging function
	
	var _logel, _logover = false, _lastlog, _lastcnt = 1;
	
	log = function(c) {
		if ( !DEBUG || _logover ) {
			return;
		}
		if ( !_logel ) {
			_logel = $('<div id="log" style="position:fixed;left:0;top:0;width:200px;bottom:0;overflow:auto;padding:10px;background-color:rgba(0,0,0,0.5);color:#fff;font-size:15px;z-index:99999"></div>').hover(function(){
				_logover = true;
			}, function(){
				_logover = false;
			}).appendTo('body');
		}
		if (c === _lastlog) {
			_logel.children(':first').empty().html(_lastlog + ' (' + (++_lastcnt) + ')');
		} else {
			$('<div style="height:2em;overflow:hidden;">' + c + '</div>').prependTo(_logel);
			_lastlog = c;
			_lastcnt = 1;
		}
	};
	
	// logEvents :: debugging events
	
	$.fn.logEvents = function( e ) {
		if ( !DEBUG ) {
			return;
		}
		
		var events = e || 'mousedown mouseup mouseover mouseout mousewheel dragstart click blur focus load unload reset submit change abort cut copy paste selection drag drop orientationchange touchstart touchmove touchend touchcancel pointerdown pointermove pointerup MSPointerDown MSPointerMove MSPointerUp gesturestart gesturechange gestureend';

		return this.each(function() {
			$(this).on(events, function(e) {
				if (typeof e === UNDEF) {
					log('Undefined event');
				} else if (e.target) {
					if (e.target.id !== 'log') { 
						log(e.type + ' <span style="padding:0 4px;font-size:0.8em;background-color:#000;border-radius:4px;"><b>' + (e.target.nodeName? e.target.nodeName.toLowerCase() : '???') + '</b>' + (e.target.id? (':'+e.target.id) : '') + '</span>' + 
							(e.relatedTarget? (' <span style="padding:0 4px;font-size:0.8em;background-color:#800;border-radius:4px;"><b>' + e.relatedTarget.nodeName.toLowerCase() + '</b>' + (e.relatedTarget.id? (':'+e.relatedTarget.id) : '') + '</span>') : ''));
					}
				} else {
					log('No event target!');
				}
				return true;
			});
		});
	};
	
	// logCss :: tracks css values until the element is live
	
	$.fn.logCss = function( p, dur, step ) {
		if ( !DEBUG ) {
			return;
		}
		
		step = step || 20;
		dur = dur || 2000;
		var t0 = new Date();
		
		return this.each(function() {
			var el = $(this);
			var show = function( nm ) {
				var t = new Date() - t0;
				log(t + '&nbsp;::&nbsp;' + nm + ' = ' + el.css(nm));
				if (t > dur) {
					clearInterval(iv);
				}
			};
			var iv = setInterval(function() {
				if ( $.isArray(p) ) {
					for (var i = 0; i < p.length; i++) {
						show(p[i]);
					}
				}
				else {
					show(p);
				}
			}, step);
		});
	};
	
})(jQuery);

/*
 *  Relative date
 */

var getRelativeDate = function(days) {
	
	if (!days)
		return translate('today');
	
	if (days===1)
		return translate('yesterday');
		
	var s, n;
	
	if (days >= 730) {
		s = translate('yearsAgo');
		n = Math.floor(days / 365);
	} else if (days >= 60) {
		s = translate('monthsAgo');
		n = Math.floor(days / 30.5);
	} else {
		s = translate('daysAgo');
		n = days;
	}
	
	return s.replace('{0}', n);
};

